<template>
  <Dialog v-model="dialogVisible" :title="dialogTitle" width="50%">
    <el-form
      ref="formRef"
      v-loading="formLoading"
      :model="formData"
      :rules="formRules"
      label-width="130px"
    >
      <el-row>
        <el-col :span="12">
          <el-form-item label="预测项" prop="outputId">
            <el-select-v2
              v-model="formData.outputId"
              :options="normalItemList || []"
              placeholder="请选择预测项"
              :props="{value:'value',label:'label',options:'children'}"
              clearable
              filterable
              :fit-input-width="false"
            />
          </el-form-item>
        </el-col>
        <el-col :span="6">
          <el-form-item label="统计方式" prop="pattern">
            <el-select v-model="formData.pattern" placeholder="请选择统计方式">
              <el-option
                v-for="dict in getStrDictOptions(DICT_TYPE.INFLUENCE_FACTOR_PATTERN)"
                :key="dict.value"
                :label="dict.label"
                :value="dict.value"
              />
            </el-select>
          </el-form-item>
        </el-col>
        <el-col :span="6">
          <el-form-item label="是否启用" prop="isEnable">
            <el-select v-model="formData.isEnable" placeholder="请选择">
              <el-option
                v-for="dict in getIntDictOptions(DICT_TYPE.COM_IS_INT)"
                :key="dict.value"
                :label="dict.label"
                :value="dict.value"
              />
            </el-select>
          </el-form-item>
        </el-col>
      </el-row>
      <el-divider content-position="left">影响因素
      </el-divider>
      <el-button
        @click="addInfluenceFactor()"
        type="primary"
        size="small">
        添加
      </el-button>
      <el-table
        :data="formData.influenceFactors"
        border
        style="width: 100%; margin-top: 5px;">
        <el-table-column prop="" label="因素预测项" align="center">
          <template #default="scope">
            <el-select-v2
              v-model="scope.row.factorOutputId"
              :options="normalItemList || []"
              placeholder="请选择"
              :props="{value:'value',label:'label',options:'children'}"
              clearable
              filterable
              :fit-input-width="false"
            />
          </template>
        </el-table-column>
        <el-table-column prop="" label="偏差值" align="center">
          <template #default="scope">
            <el-input-number v-model="scope.row.deviationValue" :min="0" clearable/>
          </template>
        </el-table-column>
        <el-table-column prop="" label="操作" width="80" align="center">
          <template #default="scope">
            <el-button
              @click="deleteInfluenceFactor(scope.$index, formData.influenceFactors)"
              type="text"
              size="small">
              删除
            </el-button>
          </template>
        </el-table-column>
      </el-table>
    </el-form>
    <template #footer>
      <el-button :disabled="formLoading" type="primary" @click="submitForm">确 定</el-button>
      <el-button @click="dialogVisible = false">取 消</el-button>
    </template>
  </Dialog>
</template>
<script lang="ts" setup>
import {DICT_TYPE, getIntDictOptions,getStrDictOptions} from "@/utils/dict";
import * as MmPredictItem from '@/api/model/pre/item'
import * as InfluenceFactorApi from '@/api/model/pre/influenceFactor/influenceFactorApi'

defineOptions({name: 'MmPredictInfluenceFactorForm'})

const {t} = useI18n() // 国际化
const message = useMessage() // 消息弹窗
const dialogVisible = ref(false) // 弹窗的是否展示
const dialogTitle = ref('') // 弹窗的标题
const formLoading = ref(false) // 表单的加载中：1）修改时的数据加载；2）提交的按钮禁用
const formType = ref('') // 表单的类型：create - 新增；update - 修改
const formData = ref({
  id: undefined,
  outputId: undefined,
  pattern: undefined,
  isEnable: 1,
  influenceFactors: []
})
const formRules = reactive({
  outputId: [{required: true, message: '预测项不能为空', trigger: 'blur'}],
  pattern: [{required: true, message: '统计方式不能为空', trigger: 'blur'}],
})
const formRef = ref() // 表单 Ref

const normalItemList = ref([])

/** 打开弹窗 */
const open = async (type: string, id?: number) => {
  dialogVisible.value = true
  dialogTitle.value = t('action.' + type)
  formType.value = type
  resetForm()

  normalItemList.value = await MmPredictItem.getNormalItemList()

  // 修改时，设置数据
  if (id) {
    formLoading.value = true
    try {
      formData.value = await InfluenceFactorApi.getInfo(id)
    } finally {
      formLoading.value = false
    }
  }

}
defineExpose({open}) // 提供 open 方法，用于打开弹窗

/** 提交表单 */
const emit = defineEmits(['success']) // 定义 success 事件，用于操作成功后的回调
const submitForm = async () => {
  // 校验表单
  if (!formRef) return
  const valid = await formRef.value.validate()
  if (!valid) return
  // 提交请求
  formLoading.value = true
  try {
    const data = formData.value
    if (formType.value === 'create') {
      await InfluenceFactorApi.create(data)
      message.success(t('common.createSuccess'))
    } else {
      await InfluenceFactorApi.update(data)
      message.success(t('common.updateSuccess'))
    }
    dialogVisible.value = false
    // 发送操作成功的事件
    emit('success')
  } finally {
    formLoading.value = false
  }
}

/** 重置表单 */
const resetForm = () => {
  formData.value = {
    id: undefined,
    outputId: undefined,
    pattern: undefined,
    isEnable: 1,
    influenceFactors: []
  }
  formRef.value?.resetFields()
}

function addInfluenceFactor() {
  if (!formData.value.influenceFactors) {
    formData.value.influenceFactors = []
  }
  formData.value.influenceFactors.push({})
}

function deleteInfluenceFactor(index: string, rows) {
  if (!rows || rows.length === 1) {
    message.error('不能全部删除!')
    return
  }
  rows.splice(index, 1)
}

</script>
